package com.reactlibrary;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.Intent;
import android.media.ThumbnailUtils;
import android.net.Uri;
import android.os.ParcelFileDescriptor;
import android.provider.MediaStore;
import android.util.Log;

import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.BaseActivityEventListener;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableNativeArray;

import java.util.Map;

import javax.annotation.Nullable;

public class RNAndroidGalleryImagePickerModule extends ReactContextBaseJavaModule {
  private final ReactApplicationContext reactContext;

  public RNAndroidGalleryImagePickerModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
    reactContext.addActivityEventListener(mActivityEventListener);
  }

  @Override
  public String getName() {
    return "RNAndroidGalleryImagePicker";
  }

  @Nullable
  @Override
  public Map<String, Object> getConstants() {
    return super.getConstants();
  }

  private static final int IMAGE_PICKER_REQUEST = 26081;

  private static final String E_ACTIVITY_DOES_NOT_EXIST = "E_ACTIVITY_DOES_NOT_EXIST";
  private static final String E_PICKER_CANCELLED = "E_PICKER_CANCELLED";
  private static final String E_FAILED_TO_SHOW_PICKER = "E_FAILED_TO_SHOW_PICKER";
  private static final String E_NO_IMAGE_DATA_FOUND = "E_NO_IMAGE_DATA_FOUND";

  private Promise mPickerPromise;

  private final ActivityEventListener mActivityEventListener = new BaseActivityEventListener() {
    @Override
    public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent intent) {
      Log.i("get activity callback", "requestCode"+requestCode+",resultCode"+resultCode);
      if (requestCode == IMAGE_PICKER_REQUEST) {
        if (mPickerPromise != null) {
          Log.i("get activity callback", "loc1");
          if (resultCode == Activity.RESULT_CANCELED) {
            mPickerPromise.reject(E_PICKER_CANCELLED, "Image picker was cancelled");
          } else if (resultCode == Activity.RESULT_OK) {
            Log.i("get activity callback", "ItemCount:" + intent.getClipData().getItemCount());
            if (intent.getClipData().getItemCount() > 0) { // intent.hasExtra(Intent.EXTRA_STREAM)) { // intent.getAction().equals(Intent.ACTION_SEND_MULTIPLE) &&
              Log.i("get activity callback", "loc2");
              WritableNativeArray imageUris = new WritableNativeArray();
              for (int itemIndex = 0;  itemIndex < intent.getClipData().getItemCount(); itemIndex++) {
                imageUris.pushString(intent.getClipData().getItemAt(itemIndex).getUri().toString());
              }
              mPickerPromise.resolve(imageUris);
//              ArrayList<Parcelable> list = intent.getParcelableArrayListExtra(Intent.EXTRA_STREAM);
//              if( list != null && list.size() > 0 ) {
//                List<String> imageUris = list.stream().map(pObj -> (Uri) pObj).map(Uri::toString).collect(Collectors.toList());
//                mPickerPromise.resolve(imageUris);
//              } else {
//                mPickerPromise.reject(E_NO_IMAGE_DATA_FOUND, "No image data found");
//              }
            } else {
              Log.i("get activity callback", "loc3");
              Uri uri = intent.getData();
              if (uri == null) {
                mPickerPromise.reject(E_NO_IMAGE_DATA_FOUND, "No image data found");
              } else {
                mPickerPromise.resolve(uri.toString());
              }
            }
          }
          mPickerPromise = null;
        }
      }
    }
  };

  @ReactMethod
  public void getContentSize(final String contentUri, final Promise promise) {
      try {
          ParcelFileDescriptor fd = getCurrentActivity().getContentResolver().openFileDescriptor(Uri.parse(contentUri), "r");
          long size = fd.getStatSize();
          fd.close();
          promise.resolve(Double.valueOf(size));
      }
      catch (Exception io) {
          promise.reject(E_ACTIVITY_DOES_NOT_EXIST, io.getMessage());
      }
  }

  @ReactMethod
  public void pickImage(final ReadableMap options, final Promise promise) {
    Log.i("RNAndroidGalleryImagePickerModule", "pickImage");

    Activity currentActivity = getCurrentActivity();

    if (currentActivity == null) {
      promise.reject(E_ACTIVITY_DOES_NOT_EXIST, "Activity doesn't exist");
      return;
    }

    // Store the promise to resolve/reject when picker returns data
    mPickerPromise = promise;

    try {
      Uri mediaUri  = MediaStore.Images.Media.EXTERNAL_CONTENT_URI;
      if (options.hasKey("video") && options.getBoolean("video")) {
        mediaUri = MediaStore.Video.Media.EXTERNAL_CONTENT_URI;
      }

      Intent intent = new Intent(Intent.ACTION_PICK, mediaUri);
      intent.putExtra(Intent.EXTRA_LOCAL_ONLY, true);
      intent.putExtra(Intent.EXTRA_ALLOW_MULTIPLE, true);
//      intent.addCategory(Intent.CATEGORY_OPENABLE);
      currentActivity.startActivityForResult(intent, IMAGE_PICKER_REQUEST);

//      MediaStore.Video.Thumbnails.getThumbnail()
//      ThumbnailUtils.createVideoThumbnail()

      /*
      final Intent galleryIntent = new Intent(); // );
      galleryIntent.setAction(Intent.ACTION_PICK); // Intent.ACTION_GET_CONTENT);
      galleryIntent.setType("image/*");
      galleryIntent.putExtra(Intent.EXTRA_LOCAL_ONLY, true);
      galleryIntent.putExtra(Intent.EXTRA_ALLOW_MULTIPLE, true);
      galleryIntent.addCategory(Intent.CATEGORY_OPENABLE);

      final Intent chooserIntent = Intent.createChooser(galleryIntent, "Pick an image ...");
      currentActivity.startActivityForResult(chooserIntent, IMAGE_PICKER_REQUEST);
      */

    } catch (Exception e) {
      mPickerPromise.reject(E_FAILED_TO_SHOW_PICKER, e);
      mPickerPromise = null;
    }
  }
}